--------------------------------------------------------
--  Table D_PLANE_DATA_SKEY_TEMP - 
--------------------------------------------------------
DROP TABLE HD_STAGGING.STG_D_PLANE_DATA_TEMP ;

CREATE TABLE HD_STAGGING.STG_D_PLANE_DATA_TEMP (  
   TAILNUM VARCHAR2(20)
  ,TYPE VARCHAR2(20)
  ,MANUFACTURER VARCHAR2(30)
  ,ISSUE_DATE VARCHAR2(10)
  ,MODEL VARCHAR2(20)
  ,STATUS VARCHAR2(20)
  ,AIRCRAFT_TYPE VARCHAR2(30)
  ,ENGINE_TYPE VARCHAR2(20)
  ,YEAR VARCHAR2(4)
 );
 
 

--------------------------------------------------------
--  Table D_PLANE_DATA_SKEY_TEMP - 
--------------------------------------------------------
DROP TABLE HD_STAGGING.STG_D_PLANE_DATA_SKEY_TEMP ;

CREATE TABLE HD_STAGGING.STG_D_PLANE_DATA_SKEY_TEMP (	
  DWH_SOURCE_KEY VARCHAR2(20)
  ,DWH_VALID_FROM DATE 
  ,YEAR VARCHAR2(4)
  ,ENGINE_TYPE VARCHAR2(20) 
  ,AIRCRAFT_TYPE VARCHAR2(30) 
  ,STATUS VARCHAR2(20)
  ,MODEL VARCHAR2(20)
  ,ISSUE_DATE VARCHAR2(10)
  ,MANUFACTURER VARCHAR2(30)
  ,TYPE VARCHAR2(20)
  ,TAILNUM VARCHAR2(20)
 ) ;

   
   
-----------------------------------------------------------------
--    STG_V_D_PLANE_DATA_TEMP_SKEY_L, dodanie Source Key to tablicy
-----------------------------------------------------------------

CREATE OR REPLACE FORCE VIEW HD_STAGGING.STG_V_D_PLANE_DATA_TEMP_SKEY_L 
(
DWH_SOURCE_KEY
,DWH_VALID_FROM
,YEAR
,ENGINE_TYPE
,AIRCRAFT_TYPE
,STATUS
,MODEL
,ISSUE_DATE
,MANUFACTURER
,TYPE
,TAILNUM
) AS SELECT 
 TAILNUM as DWH_SOURCE_KEY
,sysdate as DWH_VALID_FROM
,YEAR
,ENGINE_TYPE
,AIRCRAFT_TYPE
,STATUS
,MODEL
,ISSUE_DATE
,MANUFACTURER
,TYPE
,TAILNUM
FROM HD_STAGGING.STG_D_PLANE_DATA_TEMP;

-----------------------------------------------------------------
--    STG_D_PLANE_DATA
-----------------------------------------------------------------
DROP TABLE HD_STAGGING.STG_D_PLANE_DATA;

CREATE TABLE HD_STAGGING.STG_D_PLANE_DATA (	
   DWH_SOURCE_KEY VARCHAR2(50)
  ,DWH_DELETE_FLAG VARCHAR2(1)
  ,DWH_INSERT_FLAG VARCHAR2(1)
  ,DWH_VALID_FROM DATE
  ,YEAR VARCHAR2(4)
  ,ENGINE_TYPE VARCHAR2(20)
  ,AIRCRAFT_TYPE VARCHAR2(30)
  ,STATUS VARCHAR2(20)
  ,MODEL VARCHAR2(20)
  ,ISSUE_DATE VARCHAR2(10)
  ,MANUFACTURER VARCHAR2(30)
  ,TYPE VARCHAR2(20)
  ,TAILNUM VARCHAR2(20)
);


-----------------------------------------------------------------
--    HD_STAGGING.STG_V_D_PLANE_DATA_TEMP_L, usuniecie duplikatow,delta
-----------------------------------------------------------------
CREATE OR REPLACE FORCE VIEW HD_STAGGING.STG_V_D_PLANE_DATA_TEMP_L 
(
DWH_SOURCE_KEY
,DWH_DELETE_FLAG
,DWH_INSERT_FLAG
,DWH_VALID_FROM
,YEAR
,ENGINE_TYPE
,AIRCRAFT_TYPE
,STATUS
,MODEL
,ISSUE_DATE
,MANUFACTURER  
,TYPE
,TAILNUM
) AS
SELECT 
nvl(new.DWH_SOURCE_KEY, old.DWH_SOURCE_KEY) as DWH_SOURCE_KEY 
,CASE WHEN new.DWH_SOURCE_KEY is null AND old.DWH_DELETE_FLAG='f' THEN 't' ELSE 'f' END DWH_DELETE_FLAG
,CASE WHEN new.DWH_SOURCE_KEY is not null AND old.DWH_SOURCE_KEY is null THEN 't' ELSE 'f' END DWH_INSERT_FLAG
,new.DWH_VALID_FROM
,new.YEAR
,new.ENGINE_TYPE
,new.AIRCRAFT_TYPE
,new.STATUS
,new.MODEL
,new.ISSUE_DATE
,new.MANUFACTURER  
,new.TYPE
,new.TAILNUM
FROM 
HD_STAGGING.STG_V_D_PLANE_DATA_TEMP_SKEY_L new
-- FULL JOIN HD_STAGGING.STG_D_PLANE_DATA old ON new.DWH_SOURCE_KEY = old.DWH_SOURCE_KEY
FULL JOIN HD2.D_PLANE_DATA old ON new.DWH_SOURCE_KEY = old.DWH_SOURCE_KEY
WHERE
new.DWH_SOURCE_KEY is null AND old.DWH_DELETE_FLAG='f' OR new.DWH_SOURCE_KEY is not null AND old.DWH_SOURCE_KEY is null
group by -- remove duplicates
nvl(new.DWH_SOURCE_KEY, old.DWH_SOURCE_KEY)
,CASE WHEN new.DWH_SOURCE_KEY is null AND old.DWH_DELETE_FLAG='f' THEN 't' ELSE 'f' END 
,CASE WHEN new.DWH_SOURCE_KEY is not null AND old.DWH_SOURCE_KEY is null THEN 't' ELSE 'f' END
,new.DWH_VALID_FROM
,new.YEAR
,new.ENGINE_TYPE
,new.AIRCRAFT_TYPE
,new.STATUS
,new.MODEL
,new.ISSUE_DATE
,new.MANUFACTURER  
,new.TYPE
,new.TAILNUM;

-----------------------------------------------------------------
--    HD_STAGGING.STG_V_D_PLANE_DATA_L, modifikacja attrybuow do postaci finalnej
-----------------------------------------------------------------

CREATE OR REPLACE FORCE VIEW HD_STAGGING.STG_V_D_PLANE_DATA_L 
(
DWH_ID
,DWH_SOURCE_KEY
,DWH_DELETE_FLAG
,DWH_VALID_FROM
,FK_ISSUE_DATE
,MANUFACTURE_YEAR
,ENGINE_TYPE
,AIRCRAFT_TYPE
,STATUS
,MODEL
,ISSUE_DATE
,MANUFACTURER  
,TYPE
,TAILNUM
) AS
with 
x as(
  select nvl(max(DWH_ID),0) x 
  from HD2.D_PLANE_DATA),
m as (
SELECT 
DWH_SOURCE_KEY
,DWH_DELETE_FLAG
,DWH_VALID_FROM
,to_number(CASE WHEN REGEXP_LIKE(ISSUE_DATE,'[A-Z]') THEN null ELSE to_char(nvl(to_date(ISSUE_DATE,'MM/DD/YYYY'),to_date('01/01/1900','MM/DD/YYYY')),'YYYYMMDD') END) as FK_ISSUE_DATE
,CASE WHEN REGEXP_LIKE(YEAR,'[A-Z]') OR YEAR < =1900 THEN null ELSE trunc(to_date(YEAR,'YYYY'),'Y') END as MANUFACTURE_YEAR
,ENGINE_TYPE
,AIRCRAFT_TYPE
,STATUS
,MODEL
,CASE WHEN REGEXP_LIKE(ISSUE_DATE,'[A-Z]') THEN null ELSE nvl(to_date(ISSUE_DATE,'MM/DD/YYYY'),to_date('01/01/1900','MM/DD/YYYY')) END as ISSUE_DATE
,MANUFACTURER  
,TYPE
,TAILNUM
FROM 
HD_STAGGING.STG_D_PLANE_DATA
)
SELECT x+rownum as DWH_ID
,m.*
from x cross join m;







