--------------------------------------------------------
--  DDL for Table STG_F_FLIGHTS_TEMP
--------------------------------------------------------
drop table HD_STAGGING.STG_F_FLIGHTS_TEMP;
CREATE TABLE HD_STAGGING.STG_F_FLIGHTS_TEMP (	
FYEAR VARCHAR2(40) 
,FMONTH VARCHAR2(20)
,DAYOFMONTH VARCHAR2(20)
,DAYOFWEEK VARCHAR2(10)
,DEPTIME VARCHAR2(40)
,CRSDEPTIME VARCHAR2(40)
,ARRTIME VARCHAR2(40)
,CRSARRTIME VARCHAR2(40)
,UNIQUECARRIER VARCHAR2(20)
,FLIGHTNUM VARCHAR2(40)
,TAILNUM VARCHAR2(20)
,ACTUALELAPSEDTIME VARCHAR2(20)
,CRSELAPSEDTIME VARCHAR2(20)
,AIRTIME VARCHAR2(20)
,ARRDELAY VARCHAR2(20)
,DEPDELAY VARCHAR2(20)
,ORIGIN VARCHAR2(30)
,DEST VARCHAR2(30)
,DISTANCE VARCHAR2(30)
,TAXIIN VARCHAR2(20)
,TAXIOUT VARCHAR2(20)
,CANCELLED VARCHAR2(10)
,CANCELLATIONCODE VARCHAR2(20)
,DIVERTED VARCHAR2(10)
,CARRIERDELAY VARCHAR2(20)
,WEATHERDELAY VARCHAR2(20)
,NASDELAY VARCHAR2(20)
,SECURITYDELAY VARCHAR2(20)
,LATEAIRCRAFTDELAY VARCHAR2(20)
 );
 
 
-----------------------------------------------------------------
--    STG_V_F_FLIGHTS_SKEY_L, add Source Key to table
-----------------------------------------------------------------

CREATE OR REPLACE FORCE VIEW HD_STAGGING.STG_V_F_FLIGHTS_SKEY_L 
(
DWH_SOURCE_KEY
,DWH_VALID_FROM
,FYEAR 
,FMONTH
,DAYOFMONTH
,DAYOFWEEK
,DEPTIME
,CRSDEPTIME
,ARRTIME
,CRSARRTIME
,UNIQUECARRIER
,FLIGHTNUM
,TAILNUM
,ACTUALELAPSEDTIME
,CRSELAPSEDTIME
,AIRTIME
,ARRDELAY
,DEPDELAY
,ORIGIN
,DEST
,DISTANCE
,TAXIIN
,TAXIOUT
,CANCELLED
,CANCELLATIONCODE
,DIVERTED
,CARRIERDELAY
,WEATHERDELAY
,NASDELAY
,SECURITYDELAY
,LATEAIRCRAFTDELAY
) AS SELECT 
TAILNUM||'_'||ORIGIN||'_'||DEST||'_'||FYEAR||'_'||FMONTH||'_'||DAYOFMONTH||'_'||FLIGHTNUM||'_'||UNIQUECARRIER||'_'||TAXIIN||'_'||TAXIOUT as DWH_SOURCE_KEY
,sysdate as DWH_VALID_FROM
,FYEAR 
,FMONTH
,DAYOFMONTH
,DAYOFWEEK
,DEPTIME
,CRSDEPTIME
,ARRTIME
,CRSARRTIME
,UNIQUECARRIER
,FLIGHTNUM
,TAILNUM
,ACTUALELAPSEDTIME
,CRSELAPSEDTIME
,AIRTIME
,ARRDELAY
,DEPDELAY
,ORIGIN
,DEST
,DISTANCE
,TAXIIN
,TAXIOUT
,CANCELLED
,CANCELLATIONCODE
,DIVERTED
,CARRIERDELAY
,WEATHERDELAY
,NASDELAY
,SECURITYDELAY
,LATEAIRCRAFTDELAY
FROM HD_STAGGING.STG_F_FLIGHTS_TEMP;



-----------------------------------------------------------------
--    STG_V_STG_F_FLIGHTS_L,  remove duplicates, delta to table
-----------------------------------------------------------------

CREATE OR REPLACE FORCE VIEW HD_STAGGING.STG_V_F_FLIGHTS_L 
(
DWH_SOURCE_KEY
,DWH_DELETE_FLAG
,DWH_INSERT_FLAG
,DWH_VALID_FROM
,FYEAR 
,FMONTH
,DAYOFMONTH
,DAYOFWEEK
,DEPTIME
,CRSDEPTIME
,ARRTIME
,CRSARRTIME
,UNIQUECARRIER
,FLIGHTNUM
,TAILNUM
,ACTUALELAPSEDTIME
,CRSELAPSEDTIME
,AIRTIME
,ARRDELAY
,DEPDELAY
,ORIGIN
,DEST
,DISTANCE
,TAXIIN
,TAXIOUT
,CANCELLED
,CANCELLATIONCODE
,DIVERTED
,CARRIERDELAY
,WEATHERDELAY
,NASDELAY
,SECURITYDELAY
,LATEAIRCRAFTDELAY
) AS SELECT 
nvl(new.DWH_SOURCE_KEY, old.DWH_SOURCE_KEY) as DWH_SOURCE_KEY 
,CASE WHEN new.DWH_SOURCE_KEY is null AND old.DWH_DELETE_FLAG='f' THEN 't' ELSE 'f' END DWH_DELETE_FLAG
,CASE WHEN new.DWH_SOURCE_KEY is not null AND old.DWH_SOURCE_KEY is null THEN 't' ELSE 'f' END DWH_INSERT_FLAG
,new.DWH_VALID_FROM
,new.FYEAR 
,new.FMONTH
,new.DAYOFMONTH
,new.DAYOFWEEK
,new.DEPTIME
,new.CRSDEPTIME
,new.ARRTIME
,new.CRSARRTIME
,new.UNIQUECARRIER
,new.FLIGHTNUM
,new.TAILNUM
,new.ACTUALELAPSEDTIME
,new.CRSELAPSEDTIME
,new.AIRTIME
,new.ARRDELAY
,new.DEPDELAY
,new.ORIGIN
,new.DEST
,new.DISTANCE
,new.TAXIIN
,new.TAXIOUT
,new.CANCELLED
,new.CANCELLATIONCODE
,new.DIVERTED
,new.CARRIERDELAY
,new.WEATHERDELAY
,new.NASDELAY
,new.SECURITYDELAY
,new.LATEAIRCRAFTDELAY
FROM HD_STAGGING.STG_V_F_FLIGHTS_SKEY_L new FULL JOIN HD2.F_FLIGHTS old ON new.DWH_SOURCE_KEY = old.DWH_SOURCE_KEY
WHERE
new.DWH_SOURCE_KEY is null AND old.DWH_DELETE_FLAG='f' OR new.DWH_SOURCE_KEY is not null AND old.DWH_SOURCE_KEY is null
group by
nvl(new.DWH_SOURCE_KEY, old.DWH_SOURCE_KEY)
,CASE WHEN new.DWH_SOURCE_KEY is null AND old.DWH_DELETE_FLAG='f' THEN 't' ELSE 'f' END
,CASE WHEN new.DWH_SOURCE_KEY is not null AND old.DWH_SOURCE_KEY is null THEN 't' ELSE 'f' END
,new.DWH_VALID_FROM
,new.FYEAR 
,new.FMONTH
,new.DAYOFMONTH
,new.DAYOFWEEK
,new.DEPTIME
,new.CRSDEPTIME
,new.ARRTIME
,new.CRSARRTIME
,new.UNIQUECARRIER
,new.FLIGHTNUM
,new.TAILNUM
,new.ACTUALELAPSEDTIME
,new.CRSELAPSEDTIME
,new.AIRTIME
,new.ARRDELAY
,new.DEPDELAY
,new.ORIGIN
,new.DEST
,new.DISTANCE
,new.TAXIIN
,new.TAXIOUT
,new.CANCELLED
,new.CANCELLATIONCODE
,new.DIVERTED
,new.CARRIERDELAY
,new.WEATHERDELAY
,new.NASDELAY
,new.SECURITYDELAY
,new.LATEAIRCRAFTDELAY;



--------------------------------------------------------
--  DDL for Table STG_F_FLIGHTS
--------------------------------------------------------
drop table HD_STAGGING.STG_F_FLIGHTS;
CREATE TABLE HD_STAGGING.STG_F_FLIGHTS (	
 DWH_SOURCE_KEY VARCHAR2(50)
,DWH_DELETE_FLAG VARCHAR2(1)
,DWH_INSERT_FLAG VARCHAR2(1)
,DWH_VALID_FROM DATE
,FYEAR VARCHAR2(40) 
,FMONTH VARCHAR2(20)
,DAYOFMONTH VARCHAR2(20)
,DAYOFWEEK VARCHAR2(10)
,DEPTIME VARCHAR2(40)
,CRSDEPTIME VARCHAR2(40)
,ARRTIME VARCHAR2(40)
,CRSARRTIME VARCHAR2(40)
,UNIQUECARRIER VARCHAR2(20)
,FLIGHTNUM VARCHAR2(40)
,TAILNUM VARCHAR2(20)
,ACTUALELAPSEDTIME VARCHAR2(20)
,CRSELAPSEDTIME VARCHAR2(20)
,AIRTIME VARCHAR2(20)
,ARRDELAY VARCHAR2(20)
,DEPDELAY VARCHAR2(20)
,ORIGIN VARCHAR2(30)
,DEST VARCHAR2(30)
,DISTANCE VARCHAR2(30)
,TAXIIN VARCHAR2(20)
,TAXIOUT VARCHAR2(20)
,CANCELLED VARCHAR2(10)
,CANCELLATIONCODE VARCHAR2(20)
,DIVERTED VARCHAR2(10)
,CARRIERDELAY VARCHAR2(20)
,WEATHERDELAY VARCHAR2(20)
,NASDELAY VARCHAR2(20)
,SECURITYDELAY VARCHAR2(20)
,LATEAIRCRAFTDELAY VARCHAR2(20)
 );
 
 
 
-----------------------------------------------------------------
--    HD_STAGGING.STG_V_F_FLIGHTS_L, FK, modify attributes to final shape
-----------------------------------------------------------------

CREATE OR REPLACE FORCE VIEW HD_STAGGING.STG_V_F_FLIGHTS_HD_L 
(
DWH_ID
,DWH_SOURCE_KEY
,DWH_DELETE_FLAG
,DWH_VALID_FROM
,FK_DEST_ID
,FK_ORIGIN_ID
,FK_TAILNUM_ID
,FK_DEP_DATE_ID
,FK_CRSDEP_DATE_ID
,FK_ARR_DATE_ID
,FK_CRSARR_DATE_ID
,DEPDATETIME
,CRSDEPDATETIME
,ARRDATETIME
,CRSARRDATETIME
,UNIQUECARRIER
,FLIGHTNUM
,TAILNUM
,ACTUALELAPSEDTIME
,CRSELAPSEDTIME
,AIRTIME
,ARRDELAY
,DEPDELAY
,ORIGIN
,DEST
,DISTANCE
,TAXIIN
,TAXIOUT
,CANCELLED
,CANCELLATIONCODE
,DIVERTED
,CARRIERDELAY
,WEATHERDELAY
,NASDELAY
,SECURITYDELAY
,LATEAIRCRAFTDELAY
) AS
with 
x as(
  select nvl(max(DWH_ID),0) x 
  from HD2.F_FLIGHTS),
m as (
SELECT 
f.DWH_SOURCE_KEY
,f.DWH_DELETE_FLAG
,f.DWH_VALID_FROM
,d.DWH_ID as FK_DEST_ID
,o.DWH_ID as FK_ORIGIN_ID
,p.DWH_ID as FK_TAILNUM_ID
,to_number(
 trim(to_char(CASE WHEN REGEXP_LIKE(f.FYEAR,'[A-Z]') THEN null ELSE f.FYEAR END,'0000'))||
 trim( to_char(CASE WHEN REGEXP_LIKE(f.FMONTH,'[A-Z]') THEN null ELSE f.FMONTH END,'00'))||
 trim( to_char(CASE WHEN REGEXP_LIKE(f.DAYOFMONTH,'[A-Z]') THEN null ELSE f.DAYOFMONTH END,'00'))) as FK_DEP_DATE_ID
,to_number(
 trim(to_char(CASE WHEN REGEXP_LIKE(f.FYEAR,'[A-Z]') THEN null ELSE f.FYEAR END,'0000'))||
 trim( to_char(CASE WHEN REGEXP_LIKE(f.FMONTH,'[A-Z]') THEN null ELSE f.FMONTH END,'00'))||
 trim( to_char(CASE WHEN REGEXP_LIKE(f.DAYOFMONTH,'[A-Z]') THEN null ELSE f.DAYOFMONTH END,'00'))) as FK_CRSDEP_DATE_ID
,to_number(
 trim(to_char(CASE WHEN REGEXP_LIKE(f.FYEAR,'[A-Z]') THEN null ELSE f.FYEAR END,'0000'))||
 trim( to_char(CASE WHEN REGEXP_LIKE(f.FMONTH,'[A-Z]') THEN null ELSE f.FMONTH END,'00'))||
 trim( to_char(CASE WHEN REGEXP_LIKE(f.DAYOFMONTH,'[A-Z]') THEN null ELSE f.DAYOFMONTH END,'00'))) as FK_ARR_DATE_ID
,to_number(
 trim(to_char(CASE WHEN REGEXP_LIKE(f.FYEAR,'[A-Z]') THEN null ELSE f.FYEAR END,'0000'))||
 trim( to_char(CASE WHEN REGEXP_LIKE(f.FMONTH,'[A-Z]') THEN null ELSE f.FMONTH END,'00'))||
 trim( to_char(CASE WHEN REGEXP_LIKE(f.DAYOFMONTH,'[A-Z]') THEN null ELSE f.DAYOFMONTH END,'00'))) as FK_CRSARR_DATE_ID
,to_date(
  trim(to_char(CASE WHEN REGEXP_LIKE(f.FYEAR,'[A-Z]') THEN null ELSE f.FYEAR END,'0000'))||
  trim( to_char(CASE WHEN REGEXP_LIKE(f.FMONTH,'[A-Z]') THEN null ELSE f.FMONTH END,'00'))||
  trim( to_char(CASE WHEN REGEXP_LIKE(f.DAYOFMONTH,'[A-Z]') THEN null ELSE f.DAYOFMONTH END,'00'))||
  trim(to_char(CASE WHEN REGEXP_LIKE(f.DEPTIME,'[A-Z]') THEN null ELSE CASE when substr(f.DEPTIME,1,2)='24' THEN '00' ELSE f.DEPTIME END END,'0000')),'YYYYMMDDHH24MI') as DEPDATETIME
,to_date(
  trim(to_char(CASE WHEN REGEXP_LIKE(f.FYEAR,'[A-Z]') THEN null ELSE f.FYEAR END,'0000'))||
  trim( to_char(CASE WHEN REGEXP_LIKE(f.FMONTH,'[A-Z]') THEN null ELSE f.FMONTH END,'00'))||
  trim( to_char(CASE WHEN REGEXP_LIKE(f.DAYOFMONTH,'[A-Z]') THEN null ELSE f.DAYOFMONTH END,'00'))||
  trim(to_char(CASE WHEN REGEXP_LIKE(f.CRSDEPTIME,'[A-Z]') THEN null ELSE CASE when substr(f.CRSDEPTIME,1,2)='24' THEN '00' ELSE f.CRSDEPTIME END END,'0000')),'YYYYMMDDHH24MI') as CRSDEPDATETIME
,to_date(
  trim(to_char(CASE WHEN REGEXP_LIKE(f.FYEAR,'[A-Z]') THEN null ELSE f.FYEAR END,'0000'))||
  trim( to_char(CASE WHEN REGEXP_LIKE(f.FMONTH,'[A-Z]') THEN null ELSE f.FMONTH END,'00'))||
  trim( to_char(CASE WHEN REGEXP_LIKE(f.DAYOFMONTH,'[A-Z]') THEN null ELSE f.DAYOFMONTH END,'00'))||
  trim(to_char(CASE WHEN REGEXP_LIKE(f.ARRTIME,'[A-Z]') THEN null ELSE CASE when substr(f.ARRTIME,1,2)='24' THEN '00' ELSE f.ARRTIME END END,'0000')),'YYYYMMDDHH24MI') as ARRDATETIME
,to_date(
  trim(to_char(CASE WHEN REGEXP_LIKE(f.FYEAR,'[A-Z]') THEN null ELSE f.FYEAR END,'0000'))||
  trim( to_char(CASE WHEN REGEXP_LIKE(f.FMONTH,'[A-Z]') THEN null ELSE f.FMONTH END,'00'))||
  trim( to_char(CASE WHEN REGEXP_LIKE(f.DAYOFMONTH,'[A-Z]') THEN null ELSE f.DAYOFMONTH END,'00'))||
  trim(to_char(CASE WHEN REGEXP_LIKE(f.CRSDEPTIME,'[A-Z]') THEN null ELSE CASE when substr(f.CRSDEPTIME,1,2)='24' THEN '00' ELSE f.CRSDEPTIME END END,'0000')),'YYYYMMDDHH24MI') as CRSARRDATETIME
,f.UNIQUECARRIER
,f.FLIGHTNUM
,f.TAILNUM
,f.ACTUALELAPSEDTIME
,f.CRSELAPSEDTIME
,f.AIRTIME
,f.ARRDELAY
,f.DEPDELAY
,f.ORIGIN
,f.DEST
,f.DISTANCE
,f.TAXIIN
,f.TAXIOUT
,f.CANCELLED
,f.CANCELLATIONCODE
,f.DIVERTED
,f.CARRIERDELAY
,f.WEATHERDELAY
,f.NASDELAY
,f.SECURITYDELAY
,f.LATEAIRCRAFTDELAY 
FROM 
HD_STAGGING.STG_F_FLIGHTS f LEFT JOIN HD2.D_AIRPORT o ON f.ORIGIN = o.IATA LEFT JOIN HD2.D_AIRPORT d ON f.DEST = d.IATA LEFT JOIN HD2.D_PLANE_DATA p ON f.TAILNUM = p.TAILNUM 
)
SELECT x+rownum as DWH_ID
,m.*
from x cross join m;


